# /packages/intranet-agile/www/sprint-planner.tcl
#
# Copyright (c) 2003-2010 ]project-open[
#
# All rights reserved. Please check
# http://www.project-open.com/ for licensing details.

# Expected variables:
# project_id

set show_master_p 0
if {![info exists project_id]} {
    set show_master_p 1
    ad_page_contract {
	Show agile tasks with their status
	@author Frank Bergmann (frank.bergmann@project-open.com)
	@creation-date May 29, 2002
	@cvs-id $Id$
    } {
	project_id:integer
    }
}

# ------------------------------------------------------------
# Permissions
# ------------------------------------------------------------

set user_id [auth::require_login]
set admin_p [im_is_user_site_wide_or_intranet_admin $user_id]
im_project_permissions $user_id $project_id view read write admin
if {!$read} {
    ad_return_complaint 1 "<li>[_ intranet-core.lt_You_have_insufficient_6]"
    return
}

# ------------------------------------------------------------
# Page Title & Help Text

set page_title [lang::message::lookup "" intranet-agile.Sprint_Planner "Sprint Planner"]
set return_url [im_url_with_query]
set context_bar [im_context_bar $page_title]
set context ""

# ------------------------------------------------------------
# Defaults

set company_url "/intranet/companies/view?company_id="
set project_url "/intranet/projects/view?project_id="
set invoice_url "/intranet-invoices/view?invoice_id="
set user_url "/intranet/users/view?user_id="
set this_url [export_vars -base "/intranet-agile/sprint-planner" {project_id} ]
set sprint_planner_action_url "/intranet-agile/sprint-planner-action"


# ------------------------------------------------------------
# Determine what agile state set to display

set project_type_id [db_string ptype_id "select project_type_id from im_projects where project_id = :project_id" -default 0]
# set agile_category_type [db_string category_type_id "select aux_string1 from im_categories where category_id = :project_type_id" -default ""]
# if {"" eq $agile_category_type} { ad_return_complaint 1 "Could not determine default agile state range for project_type=[im_category_from_id $project_type_id]" }
set agile_category_type "Intranet Agile SCRUM States"
set default_agile_status_id 88110

# ------------------------------------------------------------
# Define the tasks to be displayed

set tasks_sql "
	select	t.task_id,
		t.agile_status_id,
		t.agile_sort_order,
		p.project_name as task_name,
		p.project_type_id as task_type_id,
		coalesce(p.sort_order, p.project_id) as project_sort_order,
		coalesce((
			select s.project_id from im_projects s where p.parent_id = s.project_id and s.project_type_id = [im_project_type_scrum]
		), :project_id) as sprint_id
	from	im_projects main_p,
		im_projects p,
		im_timesheet_tasks t
	where	main_p.project_id = :project_id and
		p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey) and
		p.project_id = t.task_id and
		not exists (select * from im_projects sub_p where sub_p.parent_id = p.project_id)	-- Only leafs
	order by 
	      	t.agile_sort_order
"
# ad_return_complaint 1 [im_ad_hoc_query -format "html" $tasks_sql]


# ------------------------------------------------------------
# Check that all task have a agile_status_id and agile_sort_order
#
set task_list [list 0]
db_foreach update_agile_status $tasks_sql {
    if {"" eq $agile_status_id} { db_dml uts "update im_timesheet_tasks set agile_status_id = :default_agile_status_id where task_id = :task_id" }
    if {"" eq $agile_sort_order} { db_dml uto "update im_timesheet_tasks set agile_sort_order = :project_sort_order where task_id = :task_id" }
    lappend task_list $task_id
}

# Check for duplicate sort_order
set duplicate_sort_orders [db_list dup_sort_orders "
	select	agile_sort_order
	from	(
		select	count(*) as cnt,
			t.agile_sort_order
		from	im_timesheet_tasks t
		where	t.task_id in ([join $task_list ","])
		group by t.agile_sort_order
		) t
	where	cnt > 1
"]


if {"" ne $duplicate_sort_orders} {
    # ad_return_complaint 1 "Sprint Planner: duplicate sort orders: '$duplicate_sort_orders'"
    db_dml update_dup_sort_orders "
	update im_timesheet_tasks
	set agile_sort_order = task_id
	where	task_id in ([join $task_list ","]) and
		agile_sort_order in ([join $duplicate_sort_orders ","])
    "
}

# ------------------------------------------------------------
# Calculate the sprints dimension

# Calculate sprints
set sprint_hash_sql "
	select	sprint.project_id as sprint_id,
		sprint.project_name as sprint_name
	from	im_projects main_p,
		im_projects sprint
	where	main_p.project_id = :project_id and
		sprint.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey) and
		sprint.project_type_id = [im_project_type_scrum]
	order by coalesce(sprint.sort_order, 0), sprint.project_id
"
set sprint_list [list $project_id]
db_foreach sprint_hash $sprint_hash_sql {
    lappend sprint_list $sprint_id
    set sprint_name_hash($sprint_id) $sprint_name
}

set sprint_name_hash($project_id) [acs_object_name $project_id]
# ad_return_complaint 1 "sprint_list=$sprint_list, names=[array get sprint_name_hash], order=[array get sprint_sort_order_hash]"


# ------------------------------------------------------------
# Get Top Dimension (Agile Status)
#
set top_states_sql "
	select	category_id,
		category
	from	im_categories c
	where	((c.enabled_p = 't' OR c.enabled_p is NULL) and category_type = :agile_category_type) 
		OR c.category_id in (
			select agile_status_id from ($tasks_sql) t	 
		)
	order by sort_order, category_id
"
set top_html ""
set top_states_list [list]
db_foreach top_states $top_states_sql {
    append top_html "<td class=rowtitle style='min-width:100px'>$category</td>\n"
    lappend top_states_list $category_id
}
set top_html "<tr class=rowtitle>$top_html</tr>\n"


# ------------------------------------------------------------
# Render the tasks into a cell_hash
#
set task_count 0
db_foreach tasks $tasks_sql {

    set cell ""
    set key "$sprint_id-$agile_status_id"
    if {[info exists cell_hash($key)]} { set cell $cell_hash($key) }

    switch $project_type_id {
	100 { set project_link "<a href='[export_vars -base "/intranet-timesheet2-tasks/new" {task_id}]'>$task_name</a>" }
	101 { set project_link "<a href='[export_vars -base "/intranet-helpdesk/new" {{ticket_id $task_id} {form_mode display}}]'>$task_name</a>" }
	default { set project_link "<a href='[export_vars -base "/intranet/projects/view" {{project_id $task_id}}]'>$task_name</a>" }
    }

    set color "grey"
    set left_url [export_vars -base $sprint_planner_action_url {task_id return_url {action left} project_id}]
    set right_url [export_vars -base $sprint_planner_action_url {task_id return_url {action right} project_id}]
    set up_url [export_vars -base $sprint_planner_action_url {task_id return_url {action up} project_id}]
    set down_url [export_vars -base $sprint_planner_action_url {task_id return_url {action down} project_id}]
    append cell "
	<table width=150 cellspacing=0 bgcolor=$color>
	<tr><td colspan=3 align=center><a href='$up_url'>[im_gif arrow_up]</a></td></tr>
	<tr>
	<td><a href='$left_url'>[im_gif arrow_left]</a></td>
	<td align=center>$project_link</a></td>
	<td><a href='$right_url'>[im_gif arrow_right]</a></td>
	</tr>
	<tr><td colspan=3 align=center><a href='$down_url'>[im_gif arrow_down]</a></td></tr>
	</table>
	<br>
    "
    set cell_hash($key) $cell
    incr task_count
}
# ad_return_complaint 1 [array get cell_hash]


# ------------------------------------------------------------
# Render the table body

set body_html ""
foreach sprint_id $sprint_list {

    set sprint_name $sprint_name_hash($sprint_id)
    set sprint_html "<tr class=rowtitle><td class=rowtitle colspan=999>$sprint_name</td></tr>\n"
    append sprint_html $top_html

    append sprint_html "<tr>\n"
    foreach agile_status_id $top_states_list {
	set cell ""
	set key "$sprint_id-$agile_status_id"

	if {[info exists cell_hash($key)]} { set cell $cell_hash($key) }
	append sprint_html "<td>$cell</td>"
    }
    append sprint_html "</tr>\n"

#    ad_return_complaint 1 "$sprint_html"

    append body_html $sprint_html
}


# ---------------------------------------------------------------
# Navbar
# ---------------------------------------------------------------

# Setup the subnavbar
set bind_vars [ns_set create]
ns_set put $bind_vars project_id $project_id
set show_context_help_p 0
set parent_menu_id [im_menu_id_from_label "project"]
set menu_label "agile_tasks"
set sub_navbar_html [im_sub_navbar \
    -base_url "/intranet/projects/view?project_id=$project_id" \
    $parent_menu_id \
    $bind_vars "" "pagedesriptionbar" $menu_label] 

